from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User, Group
from django.utils.translation import gettext_lazy

from django.conf import settings
from django.template.response import TemplateResponse
from django.template import RequestContext
from django.urls import reverse, NoReverseMatch
from django.utils.text import capfirst
from django.apps import apps
import datetime
import copy
from django.contrib.admin import helpers, widgets
from .fields import *
from django.db.models.fields import *
from django.db.models.fields.files import ImageField
from django.db.models import ForeignKey
from django.contrib.admin.models import LogEntry
from django.contrib.admin.utils import unquote
from django.contrib.admin.options import get_content_type_for_model
from django.shortcuts import redirect

class NotRegistered(Exception):
    pass



icon_dict = {
    '首页': 'fas fa-solid fa-house',
    '认证和授权': 'fas fa-solid fa-user',
    '用户': 'fas fa-solid fa-user',
    '组': 'fas fa-solid fa-users',
    '菜单': 'fas fa-solid fa-bars',
    '权限': 'fas fa-solid fa-key',
    '角色': 'fas fa-solid fa-user-tag',
    '日志': 'fas fa-solid fa-file-alt',
    '示例模型': 'fas fa-solid fa-file-alt',
}

class MyAdminSite(admin.AdminSite):

    # URL for the "View site" link at the top of each admin page.
    site_url = "/"

    enable_nav_sidebar = True

    # 登录页&首页的标题
    site_header = '登录页&首页的标题'
    # 浏览器的标题
    site_title = '浏览器的标题'
    # 正文的标题
    index_title = '正文的标题'
    
    index_template = 'admin/index.html'
    # index_template = 'admin/index.html'
    
    def _build_app_dict(self, request, label=None):
        """
        Build the app dictionary. The optional `label` parameter filters models
        of a specific app.
        """
        app_dict = {}

        if label:
            models = {
                m: m_a
                for m, m_a in self._registry.items()
                if m._meta.app_label == label
            }
        else:
            models = self._registry
        for model, model_admin in models.items():
            app_label = model._meta.app_label

            has_module_perms = model_admin.has_module_permission(request)
            if not has_module_perms:
                continue

            perms = model_admin.get_model_perms(request)

            # Check whether user has any perm for this module.
            # If so, add the module to the model_list.
            if True not in perms.values():
                continue

            info = (app_label, model._meta.model_name)
            model_dict = {
                "model": model,
                "name": capfirst(model._meta.verbose_name_plural),
                "object_name": model._meta.object_name,
                "perms": perms,
                "admin_url": None,
                "add_url": None,
            }
            if perms.get("change") or perms.get("view"):
                model_dict["view_only"] = not perms.get("change")
                try:
                    model_dict["admin_url"] = reverse(
                        "admin:%s_%s_changelist" % info, current_app=self.name
                    )
                except NoReverseMatch:
                    pass
            if perms.get("add"):
                try:
                    model_dict["add_url"] = reverse(
                        "admin:%s_%s_add" % info, current_app=self.name
                    )
                except NoReverseMatch:
                    pass

            if app_label in app_dict:
                app_dict[app_label]["models"].append(model_dict)
            else:
                app_dict[app_label] = {
                    "name": apps.get_app_config(app_label).verbose_name,
                    "app_label": app_label,
                    "app_url": reverse(
                        "admin:app_list",
                        kwargs={"app_label": app_label},
                        current_app=self.name,
                    ),
                    "has_module_perms": has_module_perms,
                    "models": [model_dict],
                }

        return app_dict
    
    
    def get_app_list(self, request, app_label=None):
    
        app_dict = self._build_app_dict(request, app_label)
        # Sort the apps alphabetically.
        app_list = sorted(app_dict.values(), key=lambda x: x["name"].lower())
        ## 插入首页
        app_list.insert(0, {
            'name': '首页',
            'icon': icon_dict.get('首页', '') if icon_dict.get('首页', '') else icon_dict.get('首页', ''),
            'admin_url': settings.LOGIN_REDIRECT_URL
        })
        
        # Sort the models alphabetically within each app.
        # for app in app_list:
        #     app["models"].sort(key=lambda x: x["name"])
        for app in app_list:
            app['icon'] = icon_dict.get(app['name'], '') if icon_dict.get(app['name'], '') else icon_dict.get(app['name'], '')
            if app.get('models', []):
                for model in app['models']:
                    model['icon'] = icon_dict.get(model['name'], '') if icon_dict.get(model['name'], '') else icon_dict.get(model['name'], '')

        ## 如果配置了不显示系统菜单，则只显示配置的菜单
        if not settings.SOHO_MENU_LIST['show_system_menu']:
            app_list = settings.SOHO_MENU_LIST.get('models', [])
        else:
            for sohoui_model in settings.SOHO_MENU_LIST.get('models', []):
                default_app_dict = {
                    'name': sohoui_model['name'],
                    'models': [],
                    'icon': sohoui_model.get('icon', '') if sohoui_model.get('icon', '') else  icon_dict.get(sohoui_model['name'], '')
                }
                ## 判断用户是否有权限model访问，有则添加到app_list   
                for model in sohoui_model.get('models', []):
                    if model.get('permission', ''):
                        if  request.user.has_perm(model.get('permission')):
                            default_app_dict['models'].append(model)
                    else:
                        # 判断用户是否有权限model访问，有则添加到app_list   
                        default_app_dict['models'].append(model)
                if default_app_dict['models']:
                    app_list.append(default_app_dict)
        return {
            # 站点标题
            'app_list': app_list
        }
    
    
    def each_context(self, request):
        """
        Return a dictionary of variables to put in the template context for
        *every* page in the admin site.

        For sites running on a subpath, use the SCRIPT_NAME value if site_url
        hasn't been customized.
        """
        script_name = request.META["SCRIPT_NAME"]
        site_url = (
            script_name if self.site_url == "/" and script_name else self.site_url
        )
        return {
            "site_title": self.site_title,
            "site_header": self.site_header,
            "site_url": site_url,
            "has_permission": self.has_permission(request),
            # "available_apps": self.get_app_list(request),
            "is_popup": False,
            "is_nav_sidebar_enabled": self.enable_nav_sidebar,
        }
        
    def index(self, request, extra_context=None):
        # app_list = self.get_app_list(request)
        context = {
            **self.each_context(request),
            "title": self.index_title,
            "subtitle": None,
            **(extra_context or {}),
        }
        
        context.update({
            'app_list': self.get_app_list(request)['app_list'],
        })
        context.update({
            'available_apps': context['app_list'],
        })

        request.current_app = self.name
        return TemplateResponse(
            request, self.index_template or "admin/index.html", context
        )
        
adminsite = MyAdminSite()

from django.contrib.admin.utils import (
    label_for_field,
)
def result_headers(cl):
    """
    Generate the list column headers.
    """
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(
            field_name, cl.model, model_admin=cl.model_admin, return_attr=True
        )
        yield {
            "text": text,
            "sortable": True,
        }

from django.utils.safestring import mark_safe


def get_custom_result_headers(cl):
    result_headers = []
    field_names = []
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(
            field_name, cl.model, model_admin=cl.model_admin, return_attr=True
        )
        field_names.append(field_name)
        result_headers.append(text)
    return result_headers, field_names

def get_custom_result_list(self, cl, field_names):

    model = self.model 
    
    result_list = []
    for obj, form in zip(cl.result_list, cl.formset.forms):
        result_dict = {}
        ## 获取model的change链接拼接id
        change_url = reverse('admin:%s_%s_change' % (obj._meta.app_label, obj._meta.model_name), args=[obj.id])
        result_dict['id'] = obj.id
        result_dict['change_url'] = change_url
        
        index = 1
        
        for i in items_for_result(cl, obj, form)[1:]:
            field_name = field_names[index]
            
            if hasattr(self, field_name):
                # 调用admin func
                field_value = i
            else:
            
                if field_name != '__str__':
                    if model._meta.get_field(field_name):
                        field = model._meta.get_field(field_name)
                    else:
                        field = None
                else:
                    field = None
                    
                if field_name == '__str__':
                    field_value = obj.__str__()
                
                elif hasattr(obj, 'get_%s_display' % field_name):
                    # 如果是IntegerField，直接获取字段值
                    field_value = getattr(obj, 'get_%s_display' % field_name)()
                # 判断是否ForeignKey
                elif isinstance(field, (ForeignKey, UiForeignKeyField)):
                    # 获取foreighkey的__str__
                    if getattr(obj, field_name).__str__() and getattr(obj, field_name).__str__() != 'None':
                        field_value = getattr(obj, field_name).__str__()
                    else:
                        field_value = ''
                # 判断是否BooleanField/日期格式
                elif isinstance(field, (ImageField)):
                    if getattr(obj, field_name):
                        field_value = '<img src="/static%s" style="width: 100px; height: 100px;" />' % getattr(obj, field_name).url
                    else:
                        field_value = ''
                else:
                    if isinstance(getattr(obj, field_name), bool):
                        if str(getattr(obj, field_name)) == 'True':
                            field_value = 'True'
                        elif str(getattr(obj, field_name)) == 'False':
                            field_value = 'False'
                        else:
                            field_value = str(getattr(obj, field_name))
                    elif isinstance(field, (DateField, UiDateField)):
                        if getattr(obj, field_name):
                            # 尝试解析字符串为日期
                            try:
                                field_value = getattr(obj, field_name).strftime('%Y-%m-%d')
                            except ValueError:
                                try:
                                    field_value = getattr(obj, field_name).strftime('%Y-%m-%d')
                                except ValueError:
                                    field_value = getattr(obj, field_name)
                        else:
                            field_value = ''
                    elif isinstance(field, (TimeField, UiTimeField)):
                        if getattr(obj, field_name):
                        # 尝试解析字符串为日期
                            try:
                                field_value = getattr(obj, field_name).strftime('%H:%M:%S')
                            except ValueError:
                                try:
                                    field_value = getattr(obj, field_name).strftime('%H:%M:%S')
                                except ValueError:
                                    field_value = getattr(obj, field_name)
                        else:
                            field_value = ''
                                
                    elif isinstance(field, (DateTimeField, UiDateTimeField)):
                        if getattr(obj, field_name):
                            # 尝试解析字符串为日期
                            try:
                                field_value = getattr(obj, field_name).strftime('%Y-%m-%d %H:%M:%S')
                            except ValueError:
                                try:
                                    field_value = getattr(obj, field_name).strftime('%Y-%m-%d')
                                except ValueError:
                                    field_value = getattr(obj, field_name)
                        else:
                            field_value = ''
                    else:
                        field_value = getattr(obj, field_name)
            if type(field_value) == str:
                if field_value: 
                    result_dict[field_name] = field_value
                else:
                    result_dict[field_name] = ''
            elif type(field_value) in [float, int]:
                result_dict[field_name] = field_value
            else:
                if field_value:
                    result_dict[field_name] = field_value
                else:
                    result_dict[field_name] = ''
            index += 1
        result_list.append(result_dict)
    return result_list

from django.db import models
def get_filter_choices(self, field_name):
    # 获取过滤器的选择项
    choices = []
    
    # 动态获取模型类
    model = self.model  # 当前 ModelAdmin 关联的模型
    field = model._meta.get_field(field_name)  # 获取字段对象

    if isinstance(field, models.ForeignKey):
        # 如果是外键，获取所有相关模型的实例
        related_model = field.related_model
        for obj in related_model.objects.all():
            choices.append({'value': obj.id, 'label': str(obj)})
    elif isinstance(field, models.BooleanField):
        # 如果是布尔字段，提供 True 和 False 选项
        choices = [{'value': 1, 'label': '是'}, {'value': 0, 'label': '否'}]
    elif isinstance(field, models.CharField) or isinstance(field, models.TextField) or isinstance(field, models.IntegerField):
        # 如果是字符字段，获取所有唯一值
        # 判断是否设置了choices
        if field.choices:
            if isinstance(field, models.IntegerField):
                for choice in field.choices:
                    choices.append({'value': choice[0], 'label': choice[1]})
            else:
                for choice in field.choices:
                    choices.append({'value': int(choice[0]), 'label': choice[1]})
        else:
            unique_values = model.objects.values_list(field_name, flat=True).distinct()
            for value in unique_values:
                choices.append({'value': value, 'label': value})

    return choices

def get_custom_filters(self, list_filter):
    
    # 创建一个字典来存储筛选器的键值对
    filter_list = []
    
    for filter_item in list_filter:
        # 处理字符串类型的过滤器
        if isinstance(filter_item, str):
            filter_list.append({
                'field': filter_item,
                'field_name': self.model._meta.get_field(filter_item).verbose_name, # 获取field verbose_name
                'choices': get_filter_choices(self, filter_item)
            })
        else:
            # 处理其他类型的过滤器（如元组）
            field_name = filter_item if isinstance(filter_item, tuple) else filter_item[0]
            filter_list.append({
                'field': filter_item,
                'field_name': self.model._meta.get_field(field_name).verbose_name, # 获取field verbose_name
                'choices': get_filter_choices(self, field_name)
            })
    return filter_list

from django.contrib.admin.utils import (
    display_for_field,
    display_for_value,
    get_fields_from_path,
    label_for_field,
    lookup_field,
)

from django.contrib.admin.templatetags.admin_list import ResultList, _coerce_field_name, lookup_field, display_for_value
from django.core.exceptions import ObjectDoesNotExist

def items_for_result(cl, result, form):
    """
    Generate the actual list of data.
    """
    result_list = []
    for field_index, field_name in enumerate(cl.list_display):
        empty_value_display = cl.model_admin.get_empty_value_display()
        row_classes = ["field-%s" % _coerce_field_name(field_name, field_index)]
        try:
            f, attr, value = lookup_field(field_name, result, cl.model_admin)
        except ObjectDoesNotExist:
            result_repr = empty_value_display
        else:
            empty_value_display = getattr(
                attr, "empty_value_display", empty_value_display
            )
            if f is None or f.auto_created:
                if field_name == "action_checkbox":
                    row_classes = ["action-checkbox"]
                boolean = getattr(attr, "boolean", False)
                # Set boolean for attr that is a property, if defined.
                if isinstance(attr, property) and hasattr(attr, "fget"):
                    boolean = getattr(attr.fget, "boolean", False)
                result_repr = display_for_value(value, empty_value_display, boolean)
                if isinstance(value, (datetime.date, datetime.time)):
                    row_classes.append("nowrap")
            else:
                if isinstance(f.remote_field, models.ManyToOneRel):
                    field_val = getattr(result, f.name)
                    if field_val is None:
                        result_repr = empty_value_display
                    else:
                        result_repr = field_val
                else:
                    result_repr = display_for_field(value, f, empty_value_display)
                if isinstance(
                    f, (models.DateField, models.TimeField, models.ForeignKey)
                ):
                    row_classes.append("nowrap")
        
        result_list.append(result_repr)
    return result_list
        
def results(cl):
    if cl.formset:
        for res, form in zip(cl.result_list, cl.formset.forms):
            yield ResultList(form, items_for_result(cl, res, form))
    else:
        for res in cl.result_list:
            yield ResultList(None, items_for_result(cl, res, None))
            
from django.utils.html import format_html
from django.db.models.constants import LOOKUP_SEP
from django.contrib.admin.views.main import (
    ALL_VAR,
    ORDER_VAR,
)
from django.utils.translation import gettext as _

def result_headers(cl):
    """
    Generate the list column headers.
    """
    ordering_field_columns = cl.get_ordering_field_columns()
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(
            field_name, cl.model, model_admin=cl.model_admin, return_attr=True
        )
        is_field_sortable = cl.sortable_by is None or field_name in cl.sortable_by
        if attr:
            field_name = _coerce_field_name(field_name, i)
            # Potentially not sortable

            # if the field is the action checkbox: no sorting and special class
            if field_name == "action_checkbox":
                aria_label = _("Select all objects on this page for an action")
                yield {
                    "text": mark_safe(
                        f'<input type="checkbox" id="action-toggle" '
                        f'aria-label="{aria_label}">'
                    ),
                    "class_attrib": mark_safe(' class="action-checkbox-column"'),
                    "sortable": False,
                }
                continue

            admin_order_field = getattr(attr, "admin_order_field", None)
            # Set ordering for attr that is a property, if defined.
            if isinstance(attr, property) and hasattr(attr, "fget"):
                admin_order_field = getattr(attr.fget, "admin_order_field", None)
            if not admin_order_field and LOOKUP_SEP not in field_name:
                is_field_sortable = False

        if not is_field_sortable:
            # Not sortable
            yield {
                "text": text,
                "field_name": field_name,
                "class_attrib": format_html(' class="column-{}"', field_name),
                "sortable": False,
            }
            continue

        # OK, it is sortable if we got this far
        th_classes = ["sortable", "column-{}".format(field_name)]
        order_type = ""
        new_order_type = "asc"
        sort_priority = 0
        # Is it currently being sorted on?
        is_sorted = i in ordering_field_columns
        if is_sorted:
            order_type = ordering_field_columns.get(i).lower()
            sort_priority = list(ordering_field_columns).index(i) + 1
            th_classes.append("sorted %sending" % order_type)
            new_order_type = {"asc": "desc", "desc": "asc"}[order_type]

        # build new ordering param
        o_list_primary = []  # URL for making this field the primary sort
        o_list_remove = []  # URL for removing this field from sort
        o_list_toggle = []  # URL for toggling order type for this field

        def make_qs_param(t, n):
            return ("-" if t == "desc" else "") + str(n)

        for j, ot in ordering_field_columns.items():
            if j == i:  # Same column
                param = make_qs_param(new_order_type, j)
                # We want clicking on this header to bring the ordering to the
                # front
                o_list_primary.insert(0, param)
                o_list_toggle.append(param)
                # o_list_remove - omit
            else:
                param = make_qs_param(ot, j)
                o_list_primary.append(param)
                o_list_toggle.append(param)
                o_list_remove.append(param)

        if i not in ordering_field_columns:
            o_list_primary.insert(0, make_qs_param(new_order_type, i))

        yield {
            "text": text,
            "field_name": field_name,
            "sortable": True,
            "sorted": is_sorted,
            "ascending": order_type == "asc",
            "sort_priority": sort_priority,
            "url_primary": cl.get_query_string({ORDER_VAR: ".".join(o_list_primary)}),
            "url_remove": cl.get_query_string({ORDER_VAR: ".".join(o_list_remove)}),
            "url_toggle": cl.get_query_string({ORDER_VAR: ".".join(o_list_toggle)}),
            "class_attrib": (
                format_html(' class="{}"', " ".join(th_classes)) if th_classes else ""
            ),
        }


from .fields import *
from django.core.paginator import Paginator


def pagination(cl):
    """
    Generate the series of links to the pages in a paginated list.
    """
    pagination_required = (not cl.show_all or not cl.can_show_all) and cl.multi_page
    page_range = (
        cl.paginator.get_elided_page_range(cl.page_num) if pagination_required else []
    )
    need_show_all_link = cl.can_show_all and not cl.show_all and cl.multi_page
    return {
        "cl": cl,
        "pagination_required": pagination_required,
        "show_all_url": need_show_all_link and cl.get_query_string({ALL_VAR: ""}),
        "page_range": page_range,
        "ALL_VAR": ALL_VAR,
        "1": 1,
    }
    
    
from django.contrib.admin.helpers import AdminReadonlyField
class CustomReadonlyField(AdminReadonlyField):
    def contents(self):
        field, obj, model_admin = (
            self.field["field"],
            self.form.instance,
            self.model_admin,
        )
        if field == "title":  # 替换为你的字段名
            return "自定义内容"  # 返回你想要显示的内容
        
        # 调用父类的内容处理逻辑
        return super().contents()
    
class BaseAdmin(admin.ModelAdmin):
    ## 重写DateTimeField的form_class
    
    delete_confirmation_template = 'admin/delete_selected_confirmation.html'

    def get_app_list(self, request):
        # 使用 Django 的内置方法获取 app_list
        # from django.contrib.admin import site
        return adminsite.get_app_list(request)
    
    
    def changelist_view(self, request, extra_context=None):
        # app_list = self.get_app_list(request)
        # app_list = MyAdminSite().get_app_list(request)
        # # 将 app_list 添加到上下文中
        extra_context = extra_context or {}
        # extra_context['app_list'] = app_list['app_list']
        # extra_context['available_apps'] = app_list['app_list']
        cl = self.get_changelist_instance(request)
        
        pagination_result = pagination(cl)
        
        FormSet = self.get_changelist_formset(request)
        cl.formset = FormSet(queryset=cl.result_list)
        
        # 获取admin中自定义func
        verbose_names = ','.join([str(self.model._meta.get_field(field).verbose_name) for field in self.search_fields])
        list_filters = self.get_list_filter(request)
        filter_list = get_custom_filters(self, list_filters)
        ## 获取不同app的result_headers
        result_headers1, field_names = get_custom_result_headers(cl)
        ## 获取result_list
        
        result_list = get_custom_result_list(self, cl, field_names)
        result_headers_result = []
        result_headers_index = 0
        for obj in result_headers(cl):
            if result_headers_index == 0:
                result_headers_index += 1
                continue
            
            result_headers_result.append({
                'text': obj['text'],
                'prop': obj['field_name']
            })
            result_headers_index += 1
            
        ## 获取actions
        actions = self.get_actions(request)
        actions_result = []
        for func, name, desc in actions.values():
            if name in ['delete_selected']:
                actions_result.append({
                    'name': 'delete_selected',
                    'desc': '删除选中',
                    'type': 'danger',
                    'icon': 'el-icon-delete'
                })
                continue
            actions_result.append({
                'name': name,
                'desc': desc,
                'type': func.type if hasattr(func, 'type') else 'primary',
                'icon': func.icon if hasattr(func, 'icon') else 'fa-solid fa-check'
            })

        action_failed = False
        selected = request.POST.getlist(helpers.ACTION_CHECKBOX_NAME)
        actions = self.get_actions(request)
        if (
            actions
            and request.method == "POST"
            and "index" in request.POST
            and "_save" not in request.POST
        ):
            if selected:
                
                response = self.response_action(
                    request, queryset=cl.get_queryset(request)
                )
                if response:
                    response_json  = response.__dict__
                    return TemplateResponse(
                        request,
                        self.delete_confirmation_template,
                        {
                            'context_data':{key: value.strip() if type(value) == str else value for key, value in response_json['context_data'].items()},
                            'objects_name':response_json['context_data']['objects_name']
                        }
                    )
                    return response
                
                data = request.POST.copy()
                data.pop(helpers.ACTION_CHECKBOX_NAME, None)
                data.pop("index", None)
                action_form = self.action_form(data, auto_id=None)
                action_form.fields["action"].choices = self.get_action_choices(request)

                # # If the form's valid we can handle the action.
                if action_form.is_valid():
                    action = action_form.cleaned_data["action"]
                    select_across = action_form.cleaned_data["select_across"]
                    func = self.get_actions(request)[action][0]

                    selected = request.POST.getlist(helpers.ACTION_CHECKBOX_NAME)
                    
                    queryset = cl.get_queryset(request)
                    
                    if not select_across:
                        # Perform the action only on the selected objects
                        queryset = queryset.filter(pk__in=selected)
                    
                response = func(self, request, queryset=queryset)
                
                if response:
                    return response
                else:
                    action_failed = True
            else:
                msg = _(
                    "Items must be selected in order to perform "
                    "actions on them. No items have been changed."
                )
                from django.contrib import messages
                self.message_user(request, msg, messages.WARNING)
                action_failed = True
            if action_failed:
                # Redirect back to the changelist page to avoid resubmitting the
                # form if the user refreshes the browser or uses the "No, take
                # me back" button on the action confirmation page.
                # return HttpResponseRedirect(request.get_full_path())
                from django.http import HttpResponse
                import json
                admin_messages = []
                for message in getattr(request, "_messages", []):
                    admin_messages.append({
                        'message': message.message,
                        'tags': message.tags
                    })
                    
                return HttpResponse(json.dumps({'messages': admin_messages}))
            
        if (
            actions
            and request.method == "POST"
            and helpers.ACTION_CHECKBOX_NAME in request.POST
            and "index" not in request.POST
            and "_save" not in request.POST
        ):
            if selected:
                response = self.response_action(
                    request, queryset=cl.get_queryset(request)
                )
                if response:
                    return response
                else:
                    action_failed = True
        
        extra_context.update({
            'result_list': result_list,
            'result_headers': result_headers_result,
            'filter_list': filter_list,
            'search_placeholder': verbose_names,
            'total': pagination_result['cl'].paginator.count,
            'page_size': pagination_result['cl'].list_per_page,
            'current_page': pagination_result['cl'].page_num,
            'actions_result': actions_result,
            ## 获取新增链接
            'add_url': reverse('admin:%s_%s_add' % (self.model._meta.app_label, self.model._meta.model_name)),
            'messages_info': {'tag': 'success', 'message': '操作成功！'},
            'index_url': settings.LOGIN_REDIRECT_URL,
            'has_change_permission': self.has_change_permission(request),
            'has_delete_permission': self.has_delete_permission(request),
            'has_view_permission': self.has_view_permission(request),
        })
        return super().changelist_view(request, extra_context=extra_context)
    
    def _changeform_view(self, request, object_id, form_url, extra_context):
        response = super()._changeform_view(request, object_id, form_url, extra_context)
        
        add = object_id is None
        
        try:
            for fieldset in response.context_data['adminform']:
                for line in fieldset:
                    for field in line:
                        if field.field.field.__class__.__module__ == 'django_sohoui.fields':
                            field.field.field_type = 'django_sohoui'
                        else:
                            field.field.field_type = 'django'
        except:
            print('error')
        return response

    def render_change_form(
        self, request, context, add=False, change=False, form_url="", obj=None
    ):
        fieldsets = self.get_fieldsets(request)
        fieldsets_dict = {}
        for header, fields in fieldsets:
            for field in fields['fields']:
                
                # 获取不同字段的verbose_name
                try:
                    field_name = self.model._meta.get_field(field).verbose_name
                    fieldsets_dict[field] = field_name
                except:
                    field_name = field
                    fieldsets_dict[field] = field_name
                # 判断是否存在UiImageField
                # if isinstance(self.model._meta.get_field(field), UiImageField):
                #     context['has_file_field_custom'] = True
        
        context.update({
            'back_url': reverse('admin:%s_%s_changelist' % (self.model._meta.app_label, self.model._meta.model_name)),
            'django_ui_fields': fieldsets_dict,
            'index_url': settings.LOGIN_REDIRECT_URL,
            'has_change_permission': self.has_change_permission(request),
            'has_delete_permission': self.has_delete_permission(request),
            'has_view_permission': self.has_view_permission(request),
        })
        
        
        ## 获取app_list
        app_list = self.get_app_list(request)
        context['app_list'] = app_list['app_list']
        
        ## 获取object history
        if obj:
            object_id = obj.id
            app_label = self.opts.app_label
            app_logs = (
                LogEntry.objects.filter(
                    object_id=object_id,
                    content_type=get_content_type_for_model(self.model),
                )
                .select_related()
                .order_by("action_time")
            )
            action_list = []
            for app_log in app_logs[0:20]:
                if app_log.user.get_full_name():
                    user_name =  '%s(%s)' % (app_log.user.get_username(), app_log.user.get_full_name())
                else:
                    user_name = app_log.user.get_username() 
                action_list.append({
                    'action_time': app_log.action_time.strftime('%Y-%m-%d %H:%M:%S'),
                    'user': user_name,
                    'action': app_log.get_change_message()
                })
            
            context['action_list'] = action_list
        if obj:
            field_value = getattr(obj, 'title', None)
            
       
        return super().render_change_form(request, context, add, change, form_url, obj)
    
   
    def get_form(self, request, obj=None, **kwargs):
        # 在这里传递当前用户
        form = super().get_form(request, obj, **kwargs)
        
        form.current_user = request.user  # 将当前用户传递给表单
        form.has_change_permission = self.has_change_permission(request, obj)
        form.readonly_fields = self.get_readonly_fields(request, obj)
        return form
    

# 注册用户模型

from django.contrib.auth.admin import UserAdmin

class UserAdmin(BaseAdmin, UserAdmin):
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (_("Personal info"), {"fields": ("first_name", "last_name", "email")}),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "usable_password", "password1", "password2"),
            },
        ),
    )
    
    list_display = ("username", "email", "first_name", "last_name", "is_staff")
    list_filter = ("is_staff", "is_superuser", "is_active", "groups")
    search_fields = ("username", "first_name", "last_name", "email")
    
    filter_horizontal = (
        "groups",
        "user_permissions",
    )

adminsite.register(User, UserAdmin)

# 注册组模型
class GroupAdmin(BaseAdmin):
    filter_horizontal = ['permissions']

adminsite.register(Group, GroupAdmin)



from django.db import models
from .fields import *
from .fields import UiDateTimeField
from django.db.models.fields.files import FileField

class AdminMenus(models.Model):
    
    STATUS_CHOICES = (
        ('1', '正常'),
        ('2', '禁用'),
    )
    ACTIVE_CHOICES = (
        (1, '是'),
        (2, '否'),
    )
    
    name = UiCharField(max_length=100, verbose_name='菜单名称' )
    url = UiCharField(verbose_name='菜单链接', max_length=255,)
    icon = UiCharField(max_length=100, verbose_name='菜单图标icon', null=True, blank=True,)
    parent = models.ForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, verbose_name='父级菜单')
    sort = UiIntegerField(verbose_name='排序', null=True, blank=True, default=0)
    status = models.CharField(max_length=100, default='1', choices=STATUS_CHOICES, verbose_name='状态', null=True, blank=True,)
    active = models.IntegerField(default=1, choices=ACTIVE_CHOICES, verbose_name='是否激活')
    is_show = UiSwitchField(verbose_name='是否显示', null=True, blank=True,)
    
    def __str__(self):
        return self.name


    class Meta:
        verbose_name = 'System菜单设置'
        verbose_name_plural = 'System菜单设置'
        ordering = ['sort']


## 创建一个model，需要包含所有字段类型
class ExampleModel(models.Model):
    title = UiCharField(max_length=200, default='', verbose_name='标题')
    description = UiTextField(default='', null=True, blank=True, verbose_name='描述')
    count = UiIntegerField(default=0, verbose_name='计数')
    price = UiFloatField(default=0.0, verbose_name='价格')
    amount = UiFloatField(default=0.0, verbose_name='金额')
    is_active = UiSwitchField(default=True, verbose_name='是否激活')
    birth_date = UiDateField(null=True, blank=True, verbose_name='出生日期')
    start_time = UiTimeField(null=True, blank=True, verbose_name='开始时间')
    created = UiDateTimeField(null=True, blank=True, verbose_name='创建时间')
    email = UiEmailField(default='', null=True, blank=True, verbose_name='电子邮件')
    website = UiURLField(default='', null=True, blank=True, verbose_name='网站')
    # file_test = models.FileField(upload_to='upload/', null=True, blank=True, default='', verbose_name='文件')
    image = UiImageCharField(max_length=200, default='', null=True, blank=True, verbose_name='图片')
    adminmenus = UiForeignKeyField(AdminMenus, on_delete=models.CASCADE, null=True, blank=True, verbose_name='菜单')
    # adminmenus1 = models.ForeignKey(AdminMenus, on_delete=models.CASCADE, null=True, blank=True, verbose_name='菜单')
    # test_field = models.CharField(max_length=200, default='', null=True, blank=True, verbose_name='测试字段')
    def __str__(self):
        return self.title

    class Meta:
        verbose_name = '示例模型'
        verbose_name_plural = '示例模型'
        

class ExampleItem(models.Model):
    item = UiForeignKeyField(ExampleModel, on_delete=models.CASCADE, null=True, blank=True, verbose_name='示例模型')
    title = UiCharField(max_length=200, default='', verbose_name='标题')
    price = UiFloatField(default=0.0, verbose_name='价格')
    byorder = UiIntegerField(default=0, verbose_name='排序')
    
    def __str__(self):
        return self.title

    class Meta:
        verbose_name = '示例模型item'
        verbose_name_plural = '示例模型item'

   
import os
import uuid
import time
def get_product_image_upload_path(instance, filename):
    # return os.path.join('photos', str(instance.id), filename)
    fn, ext = os.path.splitext(filename)
    if not ext:
        ext = '.jpg'
    # name = time.strftime('%y-%m/%d',time.localtime(time.time()))
    name = str(uuid.uuid4())
    return os.path.join('pimg', name[0:3], name[3:] + ext)
   
class DemoModel(models.Model):
    title = models.CharField(max_length=200, default='', verbose_name='标题')
    description = models.TextField(default='', verbose_name='描述')
    count = models.IntegerField(default=0, verbose_name='计数')
    price = models.FloatField(default=0.0, verbose_name='价格')
    amount = models.FloatField(default=0.0, verbose_name='金额')
    is_active = models.BooleanField(default=True, verbose_name='是否激活')
    birth_date = models.DateField(null=True, blank=True, verbose_name='出生日期')
    start_time = models.TimeField(null=True, blank=True, verbose_name='开始时间')
    created = models.DateTimeField(null=True, blank=True, verbose_name='创建时间')
    email = models.EmailField(default='', null=True, blank=True, verbose_name='电子邮件')
    website = models.URLField(default='', null=True, blank=True, verbose_name='网站')
    image = models.ImageField(upload_to=get_product_image_upload_path, null=True, blank=True, default='', verbose_name='图片')
    adminmenus = models.ForeignKey(AdminMenus, on_delete=models.CASCADE, null=True, blank=True, verbose_name='菜单')
    
    def __str__(self):
        return self.title
    
    class Meta:
        verbose_name = '示例模型admin'
        verbose_name_plural = '示例模型admin'


class DemoModelItem(models.Model):
    item = models.ForeignKey(DemoModel, on_delete=models.CASCADE, null=True, blank=True, verbose_name='示例模型')
    title = models.CharField(max_length=200, default='', verbose_name='标题')
    price = models.FloatField(default=0.0, verbose_name='价格')
    
    def __str__(self):
        return self.title
    
class DemoModelItem2(models.Model):
    item = models.ForeignKey(DemoModel, on_delete=models.CASCADE, null=True, blank=True, verbose_name='示例模型')
    title = models.CharField(max_length=200, default='', verbose_name='标题')
    price = models.FloatField(default=0.0, verbose_name='价格')
    
    def __str__(self):
        return self.title



from django import forms
from .models import *
# from .widgets import *
import os
from project.settings import BASE_DIR
from .widgets import ForeignKeyInput

from django.contrib.admin.helpers import AdminReadonlyField
from django_sohoui import fields as sohoui_fields
from django_sohoui import widgets as sohoui_widgets


class UIBaseForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        ## 当页面只读时，获取base_fields, 获取model所有字段，并添加对应的FormField
        for field in self._meta.model._meta.fields:
            # 判断field是否是sohoui_fields
            if field.__class__.__module__ == 'django_sohoui.fields':
                if isinstance(field, sohoui_fields.UiCharField):
                    self.base_fields[field.name] = sohoui_fields.CharFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiTextField):
                    self.base_fields[field.name] = sohoui_fields.TextFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiForeignKeyField):
                    self.base_fields[field.name] = sohoui_fields.ForeignKeyFormField(queryset=field.related_model.objects.all(), required=not field.blank)
                elif isinstance(field, sohoui_fields.UiURLField):
                    self.base_fields[field.name] = sohoui_fields.URLFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiIntegerField):
                    self.base_fields[field.name] = sohoui_fields.NumberFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiFloatField):
                    self.base_fields[field.name] = sohoui_fields.FloatFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiSwitchField):
                    self.base_fields[field.name] = sohoui_fields.SwitchFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiDateField):
                    self.base_fields[field.name] = sohoui_fields.DateFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiTimeField):
                    self.base_fields[field.name] = sohoui_fields.TimeFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiDateTimeField):
                    self.base_fields[field.name] = sohoui_fields.UiDateTimeFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiEmailField):
                    self.base_fields[field.name] = sohoui_fields.EmailFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiImageCharField):
                    self.base_fields[field.name] = sohoui_fields.ImageCharFormField(required=not field.blank)
                elif isinstance(field, sohoui_fields.UiForeignKeyField):
                    self.base_fields[field.name] = sohoui_fields.ForeignKeyFormField(required=not field.blank)
                    self.base_fields[field.name].queryset = field.related_model.objects.all()
            # else:
            #     # 根据base_fields的类型，设置对应的formfield
            #     field_type = type(field)
            #     form_field_class = getattr(forms, field_type.__name__, forms.CharField)
            #     self.base_fields[field.name] = form_field_class()
            
        super().__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].widget.field_name = self._meta.model._meta.get_field(field).verbose_name
            self.fields[field].field_name = self._meta.model._meta.get_field(field).verbose_name
            self.fields[field].widget.class_name = self._meta.model._meta.get_field(field).name
            if self.has_change_permission:
                self.fields[field].widget.el_disabled = False
                if field in self.readonly_fields:
                    self.fields[field].widget.el_disabled = True
                else:
                    self.fields[field].widget.el_disabled = False
            else:
                self.fields[field].widget.el_disabled = True
            

                
class AdminMenusForm(UIBaseForm):
   
    class Meta:
        model = AdminMenus
        fields = '__all__'
      

class ExampleModelForm(UIBaseForm):

    class Meta:
        model = ExampleModel
        fields = '__all__'
        
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
      

class DemoModelForm(UIBaseForm):
    class Meta:
        model = DemoModel
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # 设置input框的宽度，设置成200px
        for field in self.fields:
            self.fields[field].widget.attrs['style'] = 'width: 200px;'
        ## 获取提交内容
        # if self.is_bound:
        #     for field in self.fields:
        #         self.fields[field].initial = self.data.get(field)
               
               


from django.db import models
from django import forms
from .widgets import (
    TextInput, TextareaInput, NumberInput, FloatInput, SwitchInput,
    DateInput, TimeInput, EmailInput, DateTimeInputNew, ImageInput,
    ForeignKeyInput, URLInput
)

class CharFormField(forms.fields.CharField):
    
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = TextInput()
        super(CharFormField, self).__init__(*args, **kwargs)


class UiCharField(models.CharField):
    def formfield(self, **kwargs):
        defaults = {'form_class': CharFormField}
        defaults.update(kwargs)
        return super(UiCharField, self).formfield(**defaults)

class TextFormField(forms.fields.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = TextareaInput()
        super(TextFormField, self).__init__(*args, **kwargs)

class UiTextField(models.TextField):
    def formfield(self, **kwargs):
        defaults = {'form_class': TextFormField}
        defaults.update(kwargs)
        return super(UiTextField, self).formfield(**defaults)
    

class NumberFormField(forms.fields.IntegerField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = NumberInput()
        super(NumberFormField, self).__init__(*args, **kwargs)


class UiIntegerField(models.IntegerField):
    def formfield(self, **kwargs):
        defaults = {'form_class': NumberFormField}
        defaults.update(kwargs)
        return super(UiIntegerField, self).formfield(**defaults)


class FloatFormField(forms.fields.FloatField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = FloatInput()
        super(FloatFormField, self).__init__(*args, **kwargs)


class UiFloatField(models.FloatField):
    def formfield(self, **kwargs):
        defaults = {'form_class': FloatFormField}
        defaults.update(kwargs)
        return super(UiFloatField, self).formfield(**defaults)


class SwitchFormField(forms.fields.BooleanField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = SwitchInput()
        super(SwitchFormField, self).__init__(*args, **kwargs)


class UiSwitchField(models.BooleanField):
    def formfield(self, **kwargs):
        defaults = {'form_class': SwitchFormField}
        defaults.update(kwargs)
        return super(UiSwitchField, self).formfield(**defaults)


class DateFormField(forms.fields.DateField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = DateInput()
        super(DateFormField, self).__init__(*args, **kwargs)


class UiDateField(models.DateField):
    def formfield(self, **kwargs):
        defaults = {'form_class': DateFormField}
        defaults.update(kwargs)
        return super(UiDateField, self).formfield(**defaults)
    
    
class TimeFormField(forms.fields.TimeField):
    
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = TimeInput(format="%H:%M:%S")
        super(TimeFormField, self).__init__(*args, **kwargs)    


class UiTimeField(models.TimeField):
    
    def formfield(self, **kwargs):
        defaults = {'form_class': TimeFormField}
        defaults.update(kwargs)
        
        return super(UiTimeField, self).formfield(**defaults)


class UiDateTimeFormField(forms.fields.DateTimeField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = DateTimeInputNew(format="%Y/%m/%d %H:%M:%S")
        super(UiDateTimeFormField, self).__init__(*args, **kwargs)


class UiDateTimeField(models.DateTimeField):
    def formfield(self, **kwargs):
        defaults = {'form_class': UiDateTimeFormField}
        defaults.update(kwargs)
        return super(UiDateTimeField, self).formfield(**defaults)


class EmailFormField(forms.fields.EmailField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = EmailInput()
        super(EmailFormField, self).__init__(*args, **kwargs)


class UiEmailField(models.EmailField):
    def formfield(self, **kwargs):
        defaults = {'form_class': EmailFormField}
        defaults.update(kwargs)
        return super(UiEmailField, self).formfield(**defaults)

class UiImageInput(ImageInput):
    def __init__(self, *args, **kwargs):
        self.upload_to = kwargs.pop('upload_to', '')  # 获取并移除 upload_to
        super().__init__(*args, **kwargs)
        
class ImageCharFormField(forms.fields.CharField):
    
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = UiImageInput()
        super(ImageCharFormField, self).__init__(*args, **kwargs)


class UiImageCharField(models.TextField):
    def formfield(self, **kwargs):
        defaults = {'form_class': ImageCharFormField}
        defaults.update(kwargs)
        return super(UiImageCharField, self).formfield(**defaults)


class URLFormField(forms.fields.URLField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = URLInput()
        super(URLFormField, self).__init__(*args, **kwargs)


class UiURLField(models.URLField):
    def formfield(self, **kwargs):
        defaults = {'form_class': URLFormField}
        defaults.update(kwargs)
        return super(UiURLField, self).formfield(**defaults)


class UiForeignKey(ForeignKeyInput):
    def __init__(self, *args, **kwargs):
        self.field_name = kwargs.pop('label', '')  # 获取并移除 upload_to
        self.foreigh_choices = kwargs.pop('foreigh_choices', [])
        super().__init__(*args, **kwargs)
        

class ForeignKeyFormField(forms.ModelChoiceField):
    def __init__(self, *args, **kwargs):
        foreigh_choices = []
        try:
            for obj in kwargs.get('queryset'):
                foreigh_choices.append({
                    'label': obj.__str__(),
                    'value': obj.id
                })
            kwargs['widget'] = UiForeignKey(label=kwargs.pop('label', ''), foreigh_choices=foreigh_choices)
        except:
            pass
        
        super(ForeignKeyFormField, self).__init__(*args, **kwargs)


class UiForeignKeyField(models.fields.related.ForeignKey):
    def formfield(self, **kwargs):
        defaults = {'form_class': ForeignKeyFormField}
        defaults.update(kwargs)
        return super(UiForeignKeyField, self).formfield(**defaults)


"""
HTML Widget classes
"""

import copy
import datetime
import warnings
from collections import defaultdict
from graphlib import CycleError, TopologicalSorter
from itertools import chain

from django.forms.utils import flatatt, to_current_timezone
from django.templatetags.static import static
from django.utils import formats
from django.utils.dates import MONTHS
from django.utils.formats import get_format
from django.utils.html import format_html, html_safe
from django.utils.regex_helper import _lazy_re_compile
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from django.forms.renderers import get_default_renderer

__all__ = (
    "Script",
    "Media",
    "MediaDefiningClass",
    "Widget",
    "TextInput",
    "NumberInput",
    "EmailInput",
    "URLInput",
    "ColorInput",
    "SearchInput",
    "TelInput",
    "PasswordInput",
    "HiddenInput",
    "MultipleHiddenInput",
    "FileInput",
    "ClearableFileInput",
    "Textarea",
    "DateInput",
    # "DateTimeInput",
    "DateTimeInputNew",
    "TimeInput",
    "CheckboxInput",
    "Select",
    "NullBooleanSelect",
    "SelectMultiple",
    "RadioSelect",
    "CheckboxSelectMultiple",
    "MultiWidget",
    "SplitDateTimeWidget",
    "SplitHiddenDateTimeWidget",
    "SelectDateWidget",
    "TextareaInput",
    "FloatInput",
    "SwitchInput",
    "ImageInput",
    "ForeignKeyInput",
)

MEDIA_TYPES = ("css", "js")


class MediaOrderConflictWarning(RuntimeWarning):
    pass


@html_safe
class MediaAsset:
    element_template = "{path}"

    def __init__(self, path, **attributes):
        self._path = path
        self.attributes = attributes

    def __eq__(self, other):
        # Compare the path only, to ensure performant comparison in Media.merge.
        return (self.__class__ is other.__class__ and self.path == other.path) or (
            isinstance(other, str) and self._path == other
        )

    def __hash__(self):
        # Hash the path only, to ensure performant comparison in Media.merge.
        return hash(self._path)

    def __str__(self):
        return format_html(
            self.element_template,
            path=self.path,
            attributes=flatatt(self.attributes),
        )

    def __repr__(self):
        return f"{type(self).__qualname__}({self._path!r})"

    @property
    def path(self):
        """
        Ensure an absolute path.
        Relative paths are resolved via the {% static %} template tag.
        """
        if self._path.startswith(("http://", "https://", "/")):
            return self._path
        return static(self._path)


class Script(MediaAsset):
    element_template = '<script src="{path}"{attributes}></script>'

    def __init__(self, src, **attributes):
        # Alter the signature to allow src to be passed as a keyword argument.
        super().__init__(src, **attributes)


@html_safe
class Media:
    def __init__(self, media=None, css=None, js=None):
        if media is not None:
            css = getattr(media, "css", {})
            js = getattr(media, "js", [])
        else:
            if css is None:
                css = {}
            if js is None:
                js = []
        self._css_lists = [css]
        self._js_lists = [js]

    def __repr__(self):
        return "Media(css=%r, js=%r)" % (self._css, self._js)

    def __str__(self):
        return self.render()

    @property
    def _css(self):
        css = defaultdict(list)
        for css_list in self._css_lists:
            for medium, sublist in css_list.items():
                css[medium].append(sublist)
        return {medium: self.merge(*lists) for medium, lists in css.items()}

    @property
    def _js(self):
        return self.merge(*self._js_lists)

    def render(self):
        return mark_safe(
            "\n".join(
                chain.from_iterable(
                    getattr(self, "render_" + name)() for name in MEDIA_TYPES
                )
            )
        )

    def render_js(self):
        return [
            (
                path.__html__()
                if hasattr(path, "__html__")
                else format_html('<script src="{}"></script>', self.absolute_path(path))
            )
            for path in self._js
        ]

    def render_css(self):
        # To keep rendering order consistent, we can't just iterate over items().
        # We need to sort the keys, and iterate over the sorted list.
        media = sorted(self._css)
        return chain.from_iterable(
            [
                (
                    path.__html__()
                    if hasattr(path, "__html__")
                    else format_html(
                        '<link href="{}" media="{}" rel="stylesheet">',
                        self.absolute_path(path),
                        medium,
                    )
                )
                for path in self._css[medium]
            ]
            for medium in media
        )

    def absolute_path(self, path):
        """
        Given a relative or absolute path to a static asset, return an absolute
        path. An absolute path will be returned unchanged while a relative path
        will be passed to django.templatetags.static.static().
        """
        if path.startswith(("http://", "https://", "/")):
            return path
        return static(path)

    def __getitem__(self, name):
        """Return a Media object that only contains media of the given type."""
        if name in MEDIA_TYPES:
            return Media(**{str(name): getattr(self, "_" + name)})
        raise KeyError('Unknown media type "%s"' % name)

    @staticmethod
    def merge(*lists):
        """
        Merge lists while trying to keep the relative order of the elements.
        Warn if the lists have the same elements in a different relative order.

        For static assets it can be important to have them included in the DOM
        in a certain order. In JavaScript you may not be able to reference a
        global or in CSS you might want to override a style.
        """
        ts = TopologicalSorter()
        for head, *tail in filter(None, lists):
            ts.add(head)  # Ensure that the first items are included.
            for item in tail:
                if head != item:  # Avoid circular dependency to self.
                    ts.add(item, head)
                head = item
        try:
            return list(ts.static_order())
        except CycleError:
            warnings.warn(
                "Detected duplicate Media files in an opposite order: {}".format(
                    ", ".join(repr(list_) for list_ in lists)
                ),
                MediaOrderConflictWarning,
            )
            return list(dict.fromkeys(chain.from_iterable(filter(None, lists))))

    def __add__(self, other):
        combined = Media()
        combined._css_lists = self._css_lists[:]
        combined._js_lists = self._js_lists[:]
        for item in other._css_lists:
            if item and item not in self._css_lists:
                combined._css_lists.append(item)
        for item in other._js_lists:
            if item and item not in self._js_lists:
                combined._js_lists.append(item)
        return combined


def media_property(cls):
    def _media(self):
        # Get the media property of the superclass, if it exists
        sup_cls = super(cls, self)
        try:
            base = sup_cls.media
        except AttributeError:
            base = Media()

        # Get the media definition for this class
        definition = getattr(cls, "Media", None)
        if definition:
            extend = getattr(definition, "extend", True)
            if extend:
                if extend is True:
                    m = base
                else:
                    m = Media()
                    for medium in extend:
                        m += base[medium]
                return m + Media(definition)
            return Media(definition)
        return base

    return property(_media)


class MediaDefiningClass(type):
    """
    Metaclass for classes that can have media definitions.
    """

    def __new__(mcs, name, bases, attrs):
        new_class = super().__new__(mcs, name, bases, attrs)

        if "media" not in attrs:
            new_class.media = media_property(new_class)

        return new_class


class Widget(metaclass=MediaDefiningClass):
    needs_multipart_form = False  # Determines does this widget need multipart form
    is_localized = False
    is_required = False
    supports_microseconds = True
    use_fieldset = False

    def __init__(self, attrs=None):
        self.attrs = {} if attrs is None else attrs.copy()

    def __deepcopy__(self, memo):
        obj = copy.copy(self)
        obj.attrs = self.attrs.copy()
        memo[id(self)] = obj
        return obj

    @property
    def is_hidden(self):
        return self.input_type == "hidden" if hasattr(self, "input_type") else False

    def subwidgets(self, name, value, attrs=None):
        context = self.get_context(name, value, attrs)
        yield context["widget"]

    def format_value(self, value):
        """
        Return a value as it should appear when rendered in a template.
        """
        if value == "" or value is None:
            return None
        if self.is_localized:
            return formats.localize_input(value)
        return str(value)

    def get_context(self, name, value, attrs):
        try:
            field_name = self.field_name
        except:
            field_name = ''
            
        return {
            "widget": {
                "name": name,
                'field_name': field_name,
                "is_hidden": self.is_hidden,
                "required": self.is_required,
                "value": self.format_value(value),
                "attrs": self.build_attrs(self.attrs, attrs),
                "template_name": self.template_name,
            },
        }

    def render(self, name, value, attrs=None, renderer=None):
        """Render the widget as an HTML string."""
        context = self.get_context(name, value, attrs)
        # print(context)
        return self._render(self.template_name, context, renderer)

    def _render(self, template_name, context, renderer=None):
        if renderer is None:
            renderer = get_default_renderer()
        return mark_safe(renderer.render(template_name, context))

    def build_attrs(self, base_attrs, extra_attrs=None):
        """Build an attribute dictionary."""
        return {**base_attrs, **(extra_attrs or {})}

    def value_from_datadict(self, data, files, name):
        """
        Given a dictionary of data and this widget's name, return the value
        of this widget or None if it's not provided.
        """
        return data.get(name)

    def value_omitted_from_data(self, data, files, name):
        return name not in data

    def id_for_label(self, id_):
        """
        Return the HTML ID attribute of this Widget for use by a <label>, given
        the ID of the field. Return an empty string if no ID is available.

        This hook is necessary because some widgets have multiple HTML
        elements and, thus, multiple IDs. In that case, this method should
        return an ID value that corresponds to the first ID in the widget's
        tags.
        """
        return id_

    def use_required_attribute(self, initial):
        return not self.is_hidden


class Input(Widget):
    """
    Base class for all <input> widgets.
    """

    input_type = None  # Subclasses must define this.
    template_name = "sohoui_widgets/input.html"

    def __init__(self, attrs=None):
        if attrs is not None:
            attrs = attrs.copy()
            self.input_type = attrs.pop("type", self.input_type)
        super().__init__(attrs)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        # 获取当前用户是否有编辑权
        # print(context)
        context["widget"]["type"] = self.input_type
        context["widget"]["upload_to"] = getattr(self, 'upload_to', '')
        context["widget"]["el_disabled"] = getattr(self, 'el_disabled', '')
        context["widget"]["foreigh_choices"] = getattr(self, 'foreigh_choices', [])
        return context


class TextInput(Input):
    input_type = "text"
    template_name = "sohoui_widgets/text.html"
    

class NumberInput(Input):
    input_type = "number"
    template_name = "sohoui_widgets/number.html"


class EmailInput(Input):
    input_type = "email"
    template_name = "sohoui_widgets/email.html"


class URLInput(Input):
    input_type = "url"
    template_name = "sohoui_widgets/url.html"


class ColorInput(Input):
    input_type = "color"
    template_name = "sohoui_widgets/color.html"


class SearchInput(Input):
    input_type = "search"
    template_name = "sohoui_widgets/search.html"


class TelInput(Input):
    input_type = "tel"
    template_name = "sohoui_widgets/tel.html"


class PasswordInput(Input):
    input_type = "password"
    template_name = "sohoui_widgets/password.html"

    def __init__(self, attrs=None, render_value=False):
        super().__init__(attrs)
        self.render_value = render_value

    def get_context(self, name, value, attrs):
        if not self.render_value:
            value = None
        return super().get_context(name, value, attrs)


class HiddenInput(Input):
    input_type = "hidden"
    template_name = "sohoui_widgets/hidden.html"



class Textarea(Widget):
    template_name = "sohoui_widgets/textarea.html"

    def __init__(self, attrs=None):
        # Use slightly better defaults than HTML's 20x2 box
        default_attrs = {"cols": "40", "rows": "10"}
        if attrs:
            default_attrs.update(attrs)
        super().__init__(default_attrs)


class DateTimeBaseInput(TextInput):
    format_key = ""
    supports_microseconds = False

    def __init__(self, attrs=None, format=None):
        super().__init__(attrs)
        self.format = format or None

    def format_value(self, value):
        return formats.localize_input(
            value, self.format or formats.get_format(self.format_key)[0]
        )


class DateInput(DateTimeBaseInput):
    format_key = "DATE_INPUT_FORMATS"
    template_name = "sohoui_widgets/date.html"

class DateTimeInputNew(DateTimeBaseInput):
    format_key = "DATETIME_INPUT_FORMATS"
    template_name = "sohoui_widgets/datetime.html"


class TimeInput(DateTimeBaseInput):
    format_key = "TIME_INPUT_FORMATS"
    template_name = "sohoui_widgets/time.html"

class SliderInput(Textarea):
    template_name = 'widgets/slider.html'


class TextareaInput(TextInput):
    input_type = "textarea"
    template_name = "sohoui_widgets/textarea.html"


class FloatInput(TextInput):
    template_name = "sohoui_widgets/float.html"


class SwitchInput(Input):
    template_name = "sohoui_widgets/switch.html"
    
class ImageInput(Input):
    template_name = "sohoui_widgets/image.html"
    
class ForeignKeyInput(Input):
    template_name = "sohoui_widgets/foreignkey.html"



from django.contrib import admin
from django.urls import path, include
from django_sohoui import views 

urlpatterns = [
    path('dashboard/', views.dashboard, name='dashboard'),
    path('dashboard1/', views.dashboard1, name='dashboard1'),
    path('get_data/', views.get_data, name='get_data'),
    path('upload/', views.upload, name='upload'),
    path('delete_selected_confirmation/', views.delete_selected_confirmation, name='delete_selected_confirmation'), 
    path('custom_url/', views.custom_url, name='custom_url'),
    path('home/', views.home, name='home'),
    path('test/', views.test, name='test')
]


from django.conf import settings    


def custom_context(request):
    return {
        ## 可能没有设置，添加默认值             
        'LOGIN_BG_IMAGE': getattr(settings, 'SOHO_LOGIN_BG_IMAGE', '/static/django_sohoui/images/logo_bg.jpg'),
        'SOHO_SITE_IMAGE': getattr(settings, 'SOHO_SITE_IMAGE', '/static/ui_template/images/soho_logo.jpg'),
    }




from django.shortcuts import render

from django.http import JsonResponse
from django.core.paginator import Paginator

from django.contrib.auth.models import User
from django_sohoui.adminsite import UserAdmin
from django.db.models import Q

def dashboard(request):
    return render(request, 'admin/dashboard.html')

def dashboard1(request):
    return render(request, 'admin/dashboard1.html')


def get_data(request):
    
    draw = int(request.GET.get('draw', 1))
    start = int(request.GET.get('start', 0))
    length = int(request.GET.get('length', 1))
    search_value = request.GET.get('search[value]', '')
    
    # 获取数据并进行分页
    queryset = User.objects.all()  # 替换为你的查询集
    
    if search_value:
        queryset = queryset.filter(
            Q(username__icontains=search_value)
        )
        
        
    total_records = queryset.count()
    paginator = Paginator(queryset, length)
    page_number = start // length + 1
    page_obj = paginator.get_page(page_number)
    
    data = []
    
    list_display = UserAdmin.list_display
    ## 获取list_display中的字段
    for item in page_obj:
        row = []
        row.append('<input type="checkbox" name="id" value="{}">'.format(item.id))
        for field in list_display:
            row.append(getattr(item, field))
        data.append(row)
   
        
    return JsonResponse({
        'draw': draw,
        'recordsTotal': total_records,
        'recordsFiltered': total_records,  # 如果有过滤条件，更新此值
        'data': data
    })
    
## 文件导入
from django.views.decorators.csrf import csrf_exempt
from project import settings
import os
import uuid

@csrf_exempt
def upload(request):
    ## 获取上传的文件并保存到media中
    print(request.FILES)
    try:
        print(request.FILES)
        filename = request.FILES.get('file')
        fn, ext = os.path.splitext(str(filename))
        name = str(uuid.uuid4())
        if not os.path.exists('{}/static/media/{}'.format(settings.BASE_DIR, name[0:3])):
            os.makedirs('{}/static/media/{}'.format(settings.BASE_DIR, name[0:3]))
            
        with open('{}/static/media/{}'.format(settings.BASE_DIR, os.path.join(name[0:3], name[:15]+ext)), 'wb') as f:
            f.write(filename.read())
        message = '上传成功'
        file_url = os.path.join('/static/media', os.path.join(name[0:3], name[:15]+ext))
    except Exception as e:
        file_url = ''
        message = '未选中文件'
    ## 返回文件路径
    return JsonResponse({
        'code': 200,
        'message': message,
        'url': file_url
    })
    


def delete_selected_confirmation(request):
    return render(request, 'admin/delete_selected_confirmation.html')


def custom_url(request):
    return render(request, 'custom_url.html')



def home(request):

    name = [
        {"number": "125.12", "percent": "-12.32%", "description": "订单统计信息", "icon": "/ui_template/images/bg1.webp"},
        {"number": "653.33", "percent": "+42.32%", "description": "月度计划信息", "icon": "/ui_template/images/bg1.webp"},
        {"number": "125.65", "percent": "+17.32%", "description": "年度计划信息", "icon": "/ui_template/images/bg1.webp"},
        {"number": "520.43", "percent": "-10.01%", "description": "访问统计信息", "icon": "/ui_template/images/bg1.webp"},
    ]
    
    colors = ['#FFCCCB', '#ADD8E6', '#90EE90']  # 示例颜色
    
    context = {
        "subtitle": None,
        "name": name,
        "colors": colors,  # 确保颜色列表被传递
    }

    return render(request, "admin/home-dev.html", context)


def test(request):
    data = {}
    return render(request, 'test.html', data)



from setuptools import setup, find_packages

setup(
    name='django_sohoui',  # 包的名称
    version='1.3.9',  # 包的版本
    packages=find_packages(),  # 自动找到包
    install_requires=[
        'django==4.2.21',
    ],  # 依赖的其他包
    author='zack_liu',  # 作者
    author_email='liu.zhimin.2019@gmail.com',  # 作者邮箱
    description='django soho ui base',  # 简短描述
    long_description=open('README.md').read(),  # 详细描述
    long_description_content_type='text/markdown',  # 描述格式
    url='https://github.com/liuzhimin2019/django_sohoui.git',  # 项目网址
    classifiers=[  # 分类信息
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    include_package_data=True,
    python_requires='>=3',  # Python 版本要求
    django_version='4.2.21',
)



from django.contrib import admin
from .adminsite import adminsite, BaseAdmin   
from .models import *
from .forms import *
from django.db import models
from django.contrib import messages
import csv 
from io import StringIO
from django.http import HttpResponse

class AdminMenusAdmin(BaseAdmin):
    
    form = AdminMenusForm
    list_per_page = 2
    list_display = ('name', 'url', 'icon', 'parent', 'sort', 'is_show', 'status', 'active')
    list_filter = ('parent', 'is_show', 'status', 'active')
    search_fields = ('name', 'url')
    ordering = ('sort',)
    autocomplete_fields = ['parent']

adminsite.register(AdminMenus, AdminMenusAdmin)



class ExampleItemInline(admin.TabularInline):
    model = ExampleItem
    extra = 0
    ordering = ('byorder',)
    


class ExampleModelAdmin(BaseAdmin):
    
    form = ExampleModelForm
    list_per_page = 20
    
 
    @admin.display(description='图片1')
    def show_image(self, obj):
        if obj.image:
            image_list = obj.image.split(',')
            return f'<img src="{image_list[0]}" style="width: 100px; height: 100px;" />'
        else:
            return ''
        
    def updatge_is_active(self, request, queryset):
        for obj in queryset:
            if obj.is_active:
                obj.is_active = False
            else:
                obj.is_active = True
            obj.save()
        messages.success(request, '已成功提交激活/关闭信息！')
    
    updatge_is_active.short_description = '激活/关闭'
    updatge_is_active.type = 'success'
    updatge_is_active.icon = 'fas fa-solid fa-file-alt'
    
    def export_excel(self, request, queryset):
        csv_file = StringIO()
        writer = csv.writer(csv_file)
        writer.writerow(['标题', '描述', '数量', '价格', '金额'])
        for obj in queryset:
            writer.writerow([obj.title, obj.description, obj.count, obj.price, obj.amount])
        response = HttpResponse(csv_file.getvalue(), content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename=example_model.csv'
        return response
    
    export_excel.short_description = '导出Excel'
    export_excel.type = 'warning'
    export_excel.icon = 'el-icon-share'
    
    list_display = (
        'title', 'description', 'count',  'amount', 'price', 'is_active',
        'birth_date', 'start_time', 'email', 'website', 'show_image'
    )
    list_filter = ('is_active',)
    search_fields = ('title', )
    ordering = ('-price',)
    actions = ['updatge_is_active', 'export_excel']
    inlines = [ExampleItemInline, ]

adminsite.register(ExampleModel, ExampleModelAdmin)


class DemoModelItemInline(admin.TabularInline):
    model = DemoModelItem
    extra = 0
    
class DemoModelItemInline2(admin.TabularInline):
    model = DemoModelItem2
    extra = 0

class DemoModelAdmin(BaseAdmin):
    
    form = DemoModelForm
    
    def export_excel(self, request, queryset):
        return ''
    export_excel.short_description = '导出Excel'
    export_excel.type = 'success'
    list_display = (
        'title', 'description', 'count', 'price', 'amount', 'is_active', 'adminmenus', 'birth_date', 'start_time', 'email', 'website', 'image'
    )
    list_filter = ('is_active',)
    search_fields = ('title', )
    actions = ['export_excel', ]
    autocomplete_fields = ['adminmenus']
    inlines = [DemoModelItemInline, DemoModelItemInline2]

adminsite.register(DemoModel, DemoModelAdmin)


